#include <stdio.h>
#include "kv_map.h"

typedef struct {
    const char * name;
    double       val;
    const char * desc;
} math_const_t;

static int set_default(const char   * key, math_const_t * v)
{
    *v = (math_const_t) {key, 0.0, "not available"};
    return 0;
}

int main(void)
{
    // Create a map for structures of typemath_const_t
    kv_map_t * map = & KV_MAP_INIT(math_const_t, false, set_default, NULL);

    // Add a few nodes
    math_const_t * v = kv_map_add(map, "pi");
    v->val  = 3.1415926535897931;
    v->desc = "ratio of circle's circumfence to diameter";

    v = kv_map_add(map, "e");
    v->val  = 2.7182818284590451;
    v->desc = "Euler's number";

    v = kv_map_add(map, "pht");
    v->val  = 1.6180339887498945;
    v->desc = "Golden ratio";

    kv_map_add(map, "dummy");

    // Get a node and print its description string
    printf("%s\n", ((math_const_t *) kv_map_get(map, "dummy"))->desc);

    /* Create an iterator for the map */
    kv_map_iter_t * iter = & KV_MAP_ITER_INIT(map);

    // Iterate over the map, printing content of allstructures
    while ((v = kv_map_next_value(iter)))
        printf("'%s' %lf '%s'\n", v->name, v->val, v->desc);

    // Remove a node
    kv_map_remove(map, "dummy");

    // Iterate again over the map, using the same iterator
    const char * k;
    while ((k = kv_map_next_key(iter)))
        printf("%s\n", k);

    // Free all memory that was allocated for the map
    kv_map_clear(map);
}
